/*____________________________________________________________________________*\
 *

 Copyright (c) 1997 John Roy. All rights reserved.

 These sources, libraries and applications are
 FREE FOR COMMERCIAL AND NON-COMMERCIAL USE
 as long as the following conditions are adhered to.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.

 3. Redistributions of any form whatsoever and all advertising materials 
    mentioning features must contain the following
    acknowledgment:
    "This product includes software developed by John Roy
    (http://www.johnroy.com/pi3/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHORS OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.

 *____________________________________________________________________________*|
 *
 * $Source: Hello.c$
 * $Date: Sun Aug 10 06:41:30 1997$
 *
 Description:
	Get CGI Post method.
	
	Speaks HTTP/1.0
\*____________________________________________________________________________*/
/* $SourceTop:$ */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

/*____________________________________________________________________________*\
 *
 Description:
\*____________________________________________________________________________*/
extern char **_environ;

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int Error( const char *pErrorMessage, int iLen, char *pInput )
{
	const char *pCurrent;
	int iIndex;

	printf("Content-Type: text/html\n");
	printf("\n" );
	printf("<HTML>\n");
	printf("<TITLE>PostForm: Error</TITLE>\n");
	printf("<BODY>\n");
	printf("<H2>PostForm: Error</H2>\n");
	printf("%s<BR>\n", pErrorMessage);
	printf("<BR>Stdin data length:<BR><B>%d</B><BR>\n", iLen );
	if ( iLen>0 && pInput )
		{
		printf("<BR>Stdin data:<BR><B>%s</B><BR>\n", pInput );
		};
	printf("<BR>Environment variables follow:<BR>\n");
	printf("<BR><B>\n");

	for( iIndex = 0; ; iIndex++ )
		{
		pCurrent = _environ[iIndex];
		if ( !pCurrent ) { break; };

		printf( "<CODE>%s</CODE><BR>\n", pCurrent );
		};
	printf("<BR></B>\n");

	printf("</BODY>\n");

	exit( 0 );
}

/*____________________________________________________________________________*\
 *
 Function:
 Synopsis:
 Description:
\*____________________________________________________________________________*/
int main( int iArgc, const char *ppArgv[] )
{
	const char *pContentLength;
	const char *pContentType;
	int iInLen;
	char *pInputData;
	int iRLen;

	pContentLength = getenv( "CONTENT_LENGTH" );
	if ( !pContentLength )
		{ Error( "Couldn\'t read 'CONTENT_LENGTH'", 0, 0 ); };

	pContentType = getenv( "CONTENT_TYPE" );
	if ( !pContentType )
		{ Error( "Couldn\'t read 'CONTENT_TYPE'", 0, 0 ); };

	iInLen = atoi( pContentLength );
	if ( iInLen<=0 )
		{ Error( "CONTENT_LENGTH <= 0", 0, 0 ); };

	/*
	** Allocate buffer for data
	*/
	pInputData = (char *)malloc( iInLen+1 ); 

	/*
	** Read data
	*/
	iRLen = fread( pInputData, sizeof( char ), iInLen, stdin );
	if ( iRLen!=iInLen )
		{
		Error( "Wrong number of bytes read in fread()", iRLen,
			pInputData ); 
		};
	pInputData[iInLen] = '\0';

	/* --- Hello World! --- */
	printf("Content-Type: text/html\n");
	printf("\n" );
	printf("<HTML>\n");
	printf("<TITLE>CGI Post Form Program Output</TITLE>\n");
	printf("<BODY>\n");
	printf("<H1>CGI Post Form Program Output</H1>\n");
	printf("CONTENT_TYPE: <CODE>%s</CODE><BR><BR>\n", pContentType );
	printf("%d Bytes read:<BR>\n<VERBATIM>%s</VERBATIM><BR><BR>\n",
		iInLen, pInputData );
	printf("</BODY>\n");

	return 0;
}

